@@ -1,18 +1,28 @@
-/* -*- mode: js2; js2-basic-offset: 4; indent-tabs-mode: nil -*- */
-/* exported init enable disable */
+// SPDX-FileCopyrightText: 2011 Vamsi Krishna Brahmajosyula <vamsikrishna.brahmajosyula@gmail.com>
+// SPDX-FileCopyrightText: 2011 Giovanni Campagna <gcampagna@src.gnome.org>
+// SPDX-FileCopyrightText: 2013 Debarshi Ray <debarshir@gnome.org>
+// SPDX-FileCopyrightText: 2013 Florian Müllner <fmuellner@gnome.org>
+//
+// SPDX-License-Identifier: GPL-2.0-or-later
 
-const {
-    Atk, Clutter, Gio, GLib, GMenu, GObject, Gtk, Meta, Shell, St,
-} = imports.gi;
-const {EventEmitter} = imports.misc.signals;
+import Atk from 'gi://Atk';
+import Clutter from 'gi://Clutter';
+import Gio from 'gi://Gio';
+import GLib from 'gi://GLib';
+import GMenu from 'gi://GMenu';
+import GObject from 'gi://GObject';
+import Gtk from 'gi://Gtk';
+import Meta from 'gi://Meta';
+import Shell from 'gi://Shell';
+import St from 'gi://St';
+import {EventEmitter} from 'resource:///org/gnome/shell/misc/signals.js';
 
-const DND = imports.ui.dnd;
-const ExtensionUtils = imports.misc.extensionUtils;
-const Main = imports.ui.main;
-const PanelMenu = imports.ui.panelMenu;
-const PopupMenu = imports.ui.popupMenu;
+import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';
 
-const _ = ExtensionUtils.gettext;
+import * as DND from 'resource:///org/gnome/shell/ui/dnd.js';
+import * as Main from 'resource:///org/gnome/shell/ui/main.js';
+import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
+import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';
 
 const appSys = Shell.AppSystem.get_default();
 
@@ -46,11 +56,8 @@ class ApplicationMenuItem extends PopupMenu.PopupBaseMenuItem {
         this.label_actor = appLabel;
 
         let textureCache = St.TextureCache.get_default();
-        let iconThemeChangedId = textureCache.connect('icon-theme-changed',
-            this._updateIcon.bind(this));
-        this.connect('destroy', () => {
-            textureCache.disconnect(iconThemeChangedId);
-        });
+        textureCache.connectObject('icon-theme-changed',
+            () => this._updateIcon(), this);
         this._updateIcon();
 
         this._delegate = this;
@@ -117,7 +124,10 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         else
             name = _('Favorites');
 
-        this.add_child(new St.Label({text: name}));
+        const label = new St.Label({text: name});
+        this.add_child(label);
+        this.actor.label_actor = label;
+
         this.connect('motion-event', this._onMotionEvent.bind(this));
         this.connect('notify::active', this._onActiveChanged.bind(this));
     }
@@ -206,9 +216,9 @@ class CategoryMenuItem extends PopupMenu.PopupBaseMenuItem {
         this._grab?.dismiss();
         delete this._grab;
 
-        let source = event.get_source();
-        if (source instanceof St.Widget)
-            source.sync_hover();
+        const targetActor = global.stage.get_event_actor(event);
+        if (targetActor instanceof St.Widget)
+            targetActor.sync_hover();
 
         return false;
     }
@@ -247,7 +257,7 @@ class DesktopTarget extends EventEmitter {
         this._desktopDestroyedId = 0;
 
         this._windowAddedId =
-            global.window_group.connect('actor-added',
+            global.window_group.connect('child-added',
                 this._onWindowAdded.bind(this));
 
         global.get_window_actors().forEach(a => {
@@ -269,9 +279,7 @@ class DesktopTarget extends EventEmitter {
 
     _setDesktop(desktop) {
         if (this._desktop) {
-            this._desktop.disconnect(this._desktopDestroyedId);
-            this._desktopDestroyedId = 0;
-
+            this._desktop.disconnectObject(this);
             delete this._desktop._delegate;
         }
 
@@ -279,9 +287,9 @@ class DesktopTarget extends EventEmitter {
         this.emit('desktop-changed');
 
         if (this._desktop) {
-            this._desktopDestroyedId = this._desktop.connect('destroy', () => {
+            this._desktop.connectObject('destroy', () => {
                 this._setDesktop(null);
-            });
+            }, this);
             this._desktop._delegate = this;
         }
     }
@@ -321,10 +329,7 @@ class DesktopTarget extends EventEmitter {
     }
 
     destroy() {
-        if (this._windowAddedId)
-            global.window_group.disconnect(this._windowAddedId);
-        this._windowAddedId = 0;
-
+        global.window_group.disconnectObject(this);
         this._setDesktop(null);
     }
 
@@ -360,6 +365,20 @@ class DesktopTarget extends EventEmitter {
     }
 }
 
+class MainLayout extends Clutter.BoxLayout {
+    static {
+        GObject.registerClass(this);
+    }
+
+    vfunc_get_preferred_height(container, forWidth) {
+        const [mainChild] = container;
+        const [minHeight, natHeight] =
+            mainChild.get_preferred_height(forWidth);
+
+        return [minHeight, natHeight + MENU_HEIGHT_OFFSET];
+    }
+}
+
 class ApplicationsButton extends PanelMenu.Button {
     static {
         GObject.registerClass(this);
@@ -377,24 +396,23 @@ class ApplicationsButton extends PanelMenu.Button {
         this.accessible_role = Atk.Role.LABEL;
 
         this._label = new St.Label({
-            text: _('Applications'),
+            text: _('Apps'),
             y_expand: true,
             y_align: Clutter.ActorAlign.CENTER,
         });
 
-        this.add_actor(this._label);
+        this.add_child(this._label);
         this.name = 'panelApplications';
         this.label_actor = this._label;
 
-        this._showingId = Main.overview.connect('showing', () => {
-            this.add_accessible_state(Atk.StateType.CHECKED);
-        });
-        this._hidingId = Main.overview.connect('hiding', () => {
-            this.remove_accessible_state(Atk.StateType.CHECKED);
-        });
+        Main.overview.connectObject(
+            'showing', () => this.add_accessible_state(Atk.StateType.CHECKED),
+            'hiding', () => this.remove_accessible_state(Atk.StateType.CHECKED),
+            this);
+
         Main.wm.addKeybinding(
             'apps-menu-toggle-menu',
-            ExtensionUtils.getSettings(),
+            Extension.lookupByURL(import.meta.url).getSettings(),
             Meta.KeyBindingFlags.IGNORE_AUTOREPEAT,
             Shell.ActionMode.NORMAL | Shell.ActionMode.OVERVIEW,
             () => this.menu.toggle());
@@ -410,15 +428,15 @@ class ApplicationsButton extends PanelMenu.Button {
         });
 
         this._tree = new GMenu.Tree({menu_basename: 'applications.menu'});
-        this._treeChangedId = this._tree.connect('changed',
-            this._onTreeChanged.bind(this));
+        this._tree.connectObject('changed',
+            () => this._onTreeChanged(), this);
 
         this._applicationsButtons = new Map();
         this.reloadFlag = false;
         this._createLayout();
         this._display();
-        this._installedChangedId = appSys.connect('installed-changed',
-            this._onTreeChanged.bind(this));
+        appSys.connectObject('installed-changed',
+            () => this._onTreeChanged(), this);
     }
 
     _onTreeChanged() {
@@ -430,23 +448,10 @@ class ApplicationsButton extends PanelMenu.Button {
         }
     }
 
-    _createVertSeparator() {
-        let separator = new St.DrawingArea({
-            style_class: 'calendar-vertical-separator',
-            pseudo_class: 'highlighted',
-        });
-        separator.connect('repaint', this._onVertSepRepaint.bind(this));
-        return separator;
-    }
-
     _onDestroy() {
         super._onDestroy();
 
-        Main.overview.disconnect(this._showingId);
-        Main.overview.disconnect(this._hidingId);
-        appSys.disconnect(this._installedChangedId);
-        this._tree.disconnect(this._treeChangedId);
-        this._tree = null;
+        delete this._tree;
 
         Main.wm.removeKeybinding('apps-menu-toggle-menu');
 
@@ -464,21 +469,6 @@ class ApplicationsButton extends PanelMenu.Button {
         return super._onMenuKeyPress(actor, event);
     }
 
-    _onVertSepRepaint(area) {
-        let cr = area.get_context();
-        let themeNode = area.get_theme_node();
-        let [width, height] = area.get_surface_size();
-        let stippleColor = themeNode.get_color('-stipple-color');
-        let stippleWidth = themeNode.get_length('-stipple-width');
-        let x = Math.floor(width / 2) + 0.5;
-        cr.moveTo(x, 0);
-        cr.lineTo(x, height);
-        Clutter.cairo_set_source_color(cr, stippleColor);
-        cr.setDash([1, 3], 1); // Hard-code for now
-        cr.setLineWidth(stippleWidth);
-        cr.stroke();
-    }
-
     _onOpenStateChanged(menu, open) {
         if (open) {
             if (this.reloadFlag) {
@@ -524,7 +514,7 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     scrollToButton(button) {
-        let appsScrollBoxAdj = this.applicationsScrollBox.get_vscroll_bar().get_adjustment();
+        let appsScrollBoxAdj = this.applicationsScrollBox.get_vadjustment();
         let appsScrollBoxAlloc = this.applicationsScrollBox.get_allocation_box();
         let currentScrollValue = appsScrollBoxAdj.get_value();
         let boxHeight = appsScrollBoxAlloc.y2 - appsScrollBoxAlloc.y1;
@@ -539,7 +529,7 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 
     scrollToCatButton(button) {
-        let catsScrollBoxAdj = this.categoriesScrollBox.get_vscroll_bar().get_adjustment();
+        let catsScrollBoxAdj = this.categoriesScrollBox.get_vadjustment();
         let catsScrollBoxAlloc = this.categoriesScrollBox.get_allocation_box();
         let currentScrollValue = catsScrollBoxAdj.get_value();
         let boxHeight = catsScrollBoxAlloc.y2 - catsScrollBoxAlloc.y1;
@@ -556,43 +546,29 @@ class ApplicationsButton extends PanelMenu.Button {
     _createLayout() {
         let section = new PopupMenu.PopupMenuSection();
         this.menu.addMenuItem(section);
-        this.mainBox = new St.BoxLayout({vertical: false});
+        this.mainBox = new St.BoxLayout({layoutManager: new MainLayout()});
         this.leftBox = new St.BoxLayout({vertical: true});
         this.applicationsScrollBox = new St.ScrollView({
             style_class: 'apps-menu vfade',
             x_expand: true,
         });
-        this.applicationsScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
-        let vscroll = this.applicationsScrollBox.get_vscroll_bar();
-        vscroll.connect('scroll-start', () => {
-            this.menu.passEvents = true;
-        });
-        vscroll.connect('scroll-stop', () => {
-            this.menu.passEvents = false;
-        });
         this.categoriesScrollBox = new St.ScrollView({
             style_class: 'vfade',
         });
-        this.categoriesScrollBox.set_policy(Gtk.PolicyType.NEVER, Gtk.PolicyType.AUTOMATIC);
-        vscroll = this.categoriesScrollBox.get_vscroll_bar();
-        vscroll.connect('scroll-start', () => (this.menu.passEvents = true));
-        vscroll.connect('scroll-stop', () => (this.menu.passEvents = false));
         this.leftBox.add_child(this.categoriesScrollBox);
 
         this.applicationsBox = new St.BoxLayout({vertical: true});
-        this.applicationsScrollBox.add_actor(this.applicationsBox);
+        this.applicationsScrollBox.set_child(this.applicationsBox);
         this.categoriesBox = new St.BoxLayout({vertical: true});
-        this.categoriesScrollBox.add_actor(this.categoriesBox);
+        this.categoriesScrollBox.set_child(this.categoriesBox);
 
-        this.mainBox.add(this.leftBox);
-        this.mainBox.add_child(this._createVertSeparator());
+        this.mainBox.add_child(this.leftBox);
         this.mainBox.add_child(this.applicationsScrollBox);
-        section.actor.add_actor(this.mainBox);
+        section.actor.add_child(this.mainBox);
     }
 
     _display() {
         this._applicationsButtons.clear();
-        this.mainBox.style = 'width: 35em;';
         this.mainBox.hide();
 
         // Load categories
@@ -600,7 +576,7 @@ class ApplicationsButton extends PanelMenu.Button {
         this._tree.load_sync();
         let root = this._tree.get_root_directory();
         let categoryMenuItem = new CategoryMenuItem(this, null);
-        this.categoriesBox.add_actor(categoryMenuItem);
+        this.categoriesBox.add_child(categoryMenuItem);
         let iter = root.iter();
         let nextType;
         while ((nextType = iter.next()) !== GMenu.TreeItemType.INVALID) {
@@ -616,18 +592,12 @@ class ApplicationsButton extends PanelMenu.Button {
             this._loadCategory(categoryId, dir);
             if (this.applicationsByCategory[categoryId].length > 0) {
                 categoryMenuItem = new CategoryMenuItem(this, dir);
-                this.categoriesBox.add_actor(categoryMenuItem);
+                this.categoriesBox.add_child(categoryMenuItem);
             }
         }
 
         // Load applications
         this._displayButtons(this._listApplications(null));
-
-        let themeContext = St.ThemeContext.get_for_stage(global.stage);
-        let scaleFactor = themeContext.scale_factor;
-        let categoriesHeight = this.categoriesBox.height / scaleFactor;
-        let height = Math.round(categoriesHeight) + MENU_HEIGHT_OFFSET;
-        this.mainBox.style += `height: ${height}px`;
     }
 
     selectCategory(dir) {
@@ -635,7 +605,7 @@ class ApplicationsButton extends PanelMenu.Button {
             if (c._delegate instanceof PopupMenu.PopupSeparatorMenuItem)
                 c._delegate.destroy();
             else
-                this.applicationsBox.remove_actor(c);
+                this.applicationsBox.remove_child(c);
         });
 
         if (dir)
@@ -658,7 +628,7 @@ class ApplicationsButton extends PanelMenu.Button {
                 this._applicationsButtons.set(app, item);
             }
             if (!item.get_parent())
-                this.applicationsBox.add_actor(item);
+                this.applicationsBox.add_child(item);
         }
     }
 
@@ -677,22 +647,17 @@ class ApplicationsButton extends PanelMenu.Button {
     }
 }
 
-let appsMenuButton;
+export default class AppsMenuExtension extends Extension {
+    enable() {
+        this._appsMenuButton = new ApplicationsButton();
+        const index = Main.sessionMode.panel.left.indexOf('activities') + 1;
+        Main.panel.addToStatusArea(
+            'apps-menu', this._appsMenuButton, index, 'left');
+    }
 
-/** */
-function enable() {
-    appsMenuButton = new ApplicationsButton();
-    let index = Main.sessionMode.panel.left.indexOf('activities') + 1;
-    Main.panel.addToStatusArea('apps-menu', appsMenuButton, index, 'left');
-}
-
-/** */
-function disable() {
-    Main.panel.menuManager.removeMenu(appsMenuButton.menu);
-    appsMenuButton.destroy();
-}
-
-/** */
-function init() {
-    ExtensionUtils.initTranslations();
+    disable() {
+        Main.panel.menuManager.removeMenu(this._appsMenuButton.menu);
+        this._appsMenuButton.destroy();
+        delete this._appsMenuButton;
+    }
 }
